﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SilverlightSquare.Pelmanism
{
	public class Card : Control
	{
		public Card()
		{
			this.DefaultStyleKey = typeof(Card);
			this.Rank = 1;
			this.MouseEnter += new MouseEventHandler(Card_MouseEnter);
			this.MouseLeave += new MouseEventHandler(Card_MouseLeave);
		}

		void Card_MouseLeave(object sender, MouseEventArgs e) {
			VisualStateManager.GoToState(this, "Normal", true);
		}

		void Card_MouseEnter(object sender, MouseEventArgs e) {
			VisualStateManager.GoToState(this,"MouseOver",true);
		}

		public Suit Suit
		{
			get { return (Suit)GetValue(SuitProperty); }
			set { SetValue(SuitProperty, value); }
		}

		public static readonly DependencyProperty SuitProperty =
			DependencyProperty.Register("Suit", typeof(Suit), typeof(Card), new PropertyMetadata(OnSuitChanged));

		private static void OnSuitChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			((Card)sender).OnSuitChanged((Suit)e.NewValue, (Suit)e.OldValue);
		}

		private void OnSuitChanged(Suit newValue, Suit oldValue)
		{
			ChangeState();
		}

		public int Rank
		{
			get { return (int)GetValue(RankProperty); }
			set { SetValue(RankProperty, value); }
		}

		public static readonly DependencyProperty RankProperty =
			DependencyProperty.Register("Rank", typeof(int), typeof(Card), new PropertyMetadata(1,OnRankChanged));

		private static void OnRankChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			((Card)sender).OnRankChanged((int)e.NewValue, (int)e.OldValue);
		}

		private void OnRankChanged(int newValue, int oldValue)
		{
			if(newValue < 1 || newValue > 13){
				throw new InvalidOperationException();
			}
		}

		public bool IsFrontSide
		{
			get { return (bool)GetValue(IsFrontSideProperty); }
			set { SetValue(IsFrontSideProperty, value); }
		}

		public static readonly DependencyProperty IsFrontSideProperty =
			DependencyProperty.Register("IsFrontSide", typeof(bool), typeof(Card), new PropertyMetadata(OnIsFrontSideChanged));

		private static void OnIsFrontSideChanged(object sender, DependencyPropertyChangedEventArgs e)
		{
			((Card)sender).OnIsFrontSideChanged((bool)e.NewValue, (bool)e.OldValue);
		}

		private void OnIsFrontSideChanged(bool newValue, bool oldValue)
		{
			ChangeState();
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();

			ChangeState();
		}

		private void ChangeState()
		{
			VisualStateManager.GoToState(this, this.Suit.ToString(), false);
			VisualStateManager.GoToState(this, this.IsFrontSide ? "FrontSide" : "BackSide", true);
		}
	}
}
