﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SilverlightSquare.Pelmanism.Controls {
	public class SelectableItemsControl : ItemsControl {
		
		public event EventHandler<ItemSelectionChangaedEventArgs> ItemSelectionChanged;
		
		public SelectableItemsControl() {
			this.DefaultStyleKey = typeof(SelectableItemsControl);
		}

		protected override void PrepareContainerForItemOverride(DependencyObject element, object item) {
			base.PrepareContainerForItemOverride(element, item);
			SelectableItem content = element as SelectableItem;
			
			if(content == null) return;

			content.MouseLeftButtonDown += content_MouseLeftButtonDown;
		}

		private void content_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
			SelectableItem content = sender as SelectableItem;
			if(content == null){
				return;
			}
			if(content.IsLocked){
				return;
			}
			content.IsSelected = !content.IsSelected;
			if (ItemSelectionChanged != null) {
				this.ItemSelectionChanged(this,new ItemSelectionChangaedEventArgs(content));
			}
		}

		protected override bool IsItemItsOwnContainerOverride(object item) {
			return item is SelectableItem;
		}

		protected override DependencyObject GetContainerForItemOverride() {
			return new SelectableItem();
		}

		protected override void ClearContainerForItemOverride(DependencyObject element, object item) {
			SelectableItem content = element as SelectableItem;
			if (content != null){
				content.MouseLeftButtonDown -= content_MouseLeftButtonDown;
			}

			base.ClearContainerForItemOverride(element, item);
		}
		
		public void AllUnselect(){
			for(int i = 0; i < this.Items.Count; i++){
				SelectableItem item = this.ItemContainerGenerator.ContainerFromIndex(i) as SelectableItem;
				if(item != null){
					if(item.IsSelected && !item.IsLocked){
						item.IsSelected = false;
						if (ItemSelectionChanged != null) {
							this.ItemSelectionChanged(this, new ItemSelectionChangaedEventArgs(item));
						}
					}
				}
			}
		}
		
		public SelectableItem GetContainer(object item){
			return this.ItemContainerGenerator.ContainerFromItem(item) as SelectableItem;
		}
	}
	
	public class SelectableItem : ContentControl{
		public bool IsSelected {
			get { return (bool)GetValue(IsSelectedProperty); }
			set { SetValue(IsSelectedProperty, value); }
		}

		public static readonly DependencyProperty IsSelectedProperty =
			DependencyProperty.Register("IsSelected", typeof(bool), typeof(SelectableItem), new PropertyMetadata(OnIsSelectedChanged));

		private static void OnIsSelectedChanged(object sender, DependencyPropertyChangedEventArgs e) {

		}

		public bool IsLocked {
			get { return (bool)GetValue(IsLockedProperty); }
			set { SetValue(IsLockedProperty, value); }
		}

		public static readonly DependencyProperty IsLockedProperty =
			DependencyProperty.Register("IsLocked", typeof(bool), typeof(SelectableItem), new PropertyMetadata(OnIsLockedChanged));

		private static void OnIsLockedChanged(object sender, DependencyPropertyChangedEventArgs e) {

		}

	}

	public class ItemSelectionChangaedEventArgs : EventArgs {
		public ItemSelectionChangaedEventArgs(SelectableItem item){
			this.Item = item;
		}

		public SelectableItem Item { get; private set; }
	}
}
